@@ -9,7 +9,7 @@
 --
 --      guichan.lua - The main UI lua script.
 --
---      (c) Copyright 2005-2006 by François Beerten
+--      (c) Copyright 2005-2007 by François Beerten
 --
 --      This program is free software; you can redistribute it and/or modify
 --      it under the terms of the GNU General Public License as published by
@@ -25,18 +25,41 @@
 --      Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 --      02111-1307, USA.
 --
---      $Id: guichan.lua 8540 2007-02-07 19:46:20Z feb $
+--      $Id: guichan.lua 9066 2007-08-25 13:28:13Z feb $
 
 -- Global useful objects for menus  ----------
-dark = Color(38, 38, 78, 130)
-clear = Color(200, 200, 120)
+dark = Color(38, 38, 78, 128)
+darkNoAlpha = Color(38, 38, 78)
+clear = Color(200, 200, 128)
 black = Color(0, 0, 0)
+disabled = Color(112, 112, 112, 128)
 
 bckground = CGraphic:New("graphics/screens/menu.png")
 bckground:Load()
 bckground:Resize(Video.Width, Video.Height)
 backgroundWidget = ImageWidget(bckground)
 
+local SavedGame = false
+
+function FilterList(originallist, pattern)
+  local filteredlist = {}
+  local k, v
+  for k, v in ipairs(originallist) do
+    if (string.find(v, pattern)) then
+      table.insert(filteredlist, v)
+    end
+  end
+  return filteredlist  
+end
+
+function CreateFilteringLister(pattern, lister)
+  function ListFilteredItems(path)
+     return FilterList(lister(path), pattern)
+  end
+  return ListFilteredItems
+end
+
+
 function AddMenuHelpers(menu)
   function menu:addCentered(widget, x, y)
     self:add(widget, x - widget:getWidth() / 2, y)
@@ -86,6 +109,7 @@ function AddMenuHelpers(menu)
     b:setSize(size[1], size[2])
     b:setBackgroundColor(dark)
     b:setBaseColor(dark)
+    b:setDisabledColor(disabled)
     self:add(b, x, y)
     return b
   end
@@ -117,34 +141,103 @@ function AddMenuHelpers(menu)
     return bq
   end
 
-  function menu:addBrowser(path, filter, x, y, w, h, lister)
-    local mapslist = {}
-    local u = 1
-    local fileslist
-    local i
-    local f
-    if lister == nil then
-       lister = ListFilesInDirectory
+  function menu:addBrowser(path, lister, x, y, w, h, default)
+    local bq = self:addListBox(x, y, w, h, {})
+    bq.origpath = path
+    bq.actioncb = nil
+
+    local function updateList()
+      bq.itemslist = lister(bq.path)
+      if (bq.path ~= bq.origpath) then
+        table.insert(bq.itemslist, 1, "../")
+      end
+      bq:setList(bq.itemslist)
     end
-    fileslist = lister(path)
-    for i,f in ipairs(fileslist) do
-      if (string.find(f, filter)) then
-        mapslist[u] = f
-        u = u + 1
+
+    -- Change to the default directory and select the default file
+    if (default == nil) then
+      bq.path = path
+      updateList()
+    else
+      local i
+      for i = string.len(default) - 1, 1, -1 do
+        if (string.sub(default, i, i) == "/") then
+          bq.path = string.sub(default, 1, i)
+          updateList()
+
+          local f = string.sub(default, i + 1)
+          for i = 1, table.getn(bq.itemslist) do
+            if (bq.itemslist[i] == f) then
+              bq:setSelected(i - 1)
+            end
+          end
+          break
+        end
       end
     end
 
-    local bq = self:addListBox(x, y, w, h, mapslist)
-    bq.getSelectedItem = function(self)
+    function bq:getSelectedItem()
       if (self:getSelected() < 0) then
         return self.itemslist[1]
       end
       return self.itemslist[self:getSelected() + 1]
     end
 
+    -- If a directory was clicked change dirs
+    -- Otherwise call the user's callback
+    local function cb(s)
+      local f = bq:getSelectedItem()
+      if (f == "../") then
+        local i
+        for i = string.len(bq.path) - 1, 1, -1 do
+          if (string.sub(bq.path, i, i) == "/") then
+            bq.path = string.sub(bq.path, 1, i)
+            updateList()
+            break
+          end
+        end
+      elseif (string.sub(f, string.len(f)) == '/') then
+        bq.path = bq.path .. f
+        updateList()
+      else
+        if (bq.actioncb ~= nil) then
+          bq:actioncb(s)
+        end
+      end
+    end
+    bq:setActionCallback(cb)
+
+    bq.oldSetActionCallback = bq.setActionCallback
+    function bq:setActionCallback(cb)
+      bq.actioncb = cb
+    end
+
     return bq
   end
 
+  function menu:addMapBrowser(path, x, y, w, h, default)
+    local function listFilesAndDirs(path)
+        local dirlist = {}
+        local dirs = ListDirsInDirectory(path)
+        for i,f in ipairs(dirs) do
+          if (string.find(f, "^%w.*%.map$")) then
+            table.insert(dirlist, f)
+          else
+            if (string.find(f, "^%a")) then
+              table.insert(dirlist, f .. "/")
+            end
+          end
+        end
+        return dirlist
+    end
+    local browser = self:addBrowser(path, listFilesAndDirs, x, y, w, h, default)
+    local function getMap(browser)
+       return browser:getSelectedItem() .. "/presentation.smp"
+    end
+    browser.getSelectedMap = getMap
+    return browser
+  end
+
   function menu:addCheckBox(caption, x, y, callback)
     local b = CheckBox(caption)
     b:setBaseColor(clear)
@@ -171,9 +264,9 @@ function AddMenuHelpers(menu)
     dd:setFont(Fonts["game"])
     dd:setList(list)
     dd:setActionCallback(function(s) callback(dd, s) end)
-    dd:setBaseColor(dark)
+    dd:setBaseColor(darkNoAlpha)
     dd:setForegroundColor(clear)
-    dd:setBackgroundColor(dark)
+    dd:setBackgroundColor(darkNoAlpha)
     self:add(dd, x, y)
     return dd
   end
@@ -259,12 +352,12 @@ function RunResultsMenu()
   menu:writeText(_("Razings"), sx*12, sy*7)
 
   for i=0,7 do
-    if (GetPlayerData(i, "TotalUnits") > 0 ) then
-      menu:writeText(i .. " ".. GetPlayerData(i, "Name"), sx*3, sy*(8+i))
-      menu:writeText(GetPlayerData(i, "TotalUnits"), sx*6, sy*(8+i))
-      menu:writeText(GetPlayerData(i, "TotalBuildings"), sx*8, sy*(8+i))
-      menu:writeText(GetPlayerData(i, "TotalKills"), sx*10, sy*(8+i))
-      menu:writeText(GetPlayerData(i, "TotalRazings"), sx*12, sy*(8+i))     
+    if (Players[i].TotalUnits > 0) then
+      menu:writeText(i .. " ".. Players[i].Name, sx*3, sy*(8+i))
+      menu:writeText(Players[i].TotalUnits, sx*6, sy*(8+i))
+      menu:writeText(Players[i].TotalBuildings, sx*8, sy*(8+i))
+      menu:writeText(Players[i].TotalKills, sx*10, sy*(8+i))
+      menu:writeText(Players[i].TotalRazings, sx*12, sy*(8+i))     
     end
   end
 
@@ -276,9 +369,9 @@ end
 
 function RunMap(map, objective, fow, revealmap)
   if objective == nil then
-    current_objective = default_objective
+    SetObjectives(default_objective)
   else
-    current_objective = objective
+    SetObjectives(objective)
   end
   loop = true
   while (loop) do
@@ -297,11 +390,58 @@ function RunMap(map, objective, fow, revealmap)
   RunResultsMenu()
 end
 
+current_objectives = ""
+function ClearObjectives()
+  current_objectives = ""
+end
+
+function AddObjective(objective)
+  if (current_objectives ~= "") then
+    current_objectives = current_objective .. "\n" .. objective
+  else
+    current_objectives = objective
+  end
+end
+
+function SetObjectives(objectives)
+  current_objectives = objectives
+end
+
+function GetObjectives()
+  return current_objectives
+end
+
+
+-- Callback right before the game starts
+function GameStarting()
+  local i
+  local factor
+
+  if (not SavedGame) then
+    for i=0,7 do
+      -- resources can be 1,3,5 - low,medium,high
+      if (GameSettings.Resources == 1) then
+        factor = 0.5
+      elseif (GameSettings.Resources == 3) then
+        factor = 1
+      else
+        factor = 10
+      end
+      Players[i].MagmaStored = Players[i].MagmaStored * factor
+      Players[i].EnergyStored = Players[i].EnergyStored * factor
+    end
+  end
+
+  -- FIXME: get the version from somewhere else
+  UI.StatusLine:Set("Bos Wars V" .. "2.4.0" ..
+    ", (c) 1998-2007 by the Bos Wars and Stratagus Project.")
+end
+
 
 function ResetMapOptions()
-  GameSettings.Difficulty = 5
-  GameSettings.MapRichness = 5
-  GameSettings.Resources = 5
+  GameSettings.Resources = 3
+  GameSettings.Difficulty = 3
+  GameSettings.GameType = SettingsGameTypeMapDefault
   GameSettings.NoFogOfWar = false
   GameSettings.RevealMap = 0
 end
@@ -315,7 +455,8 @@ function RunStartGameMenu(s)
   local players
   local sx = Video.Width / 20
   local sy = Video.Height / 20
-  local selectedmap = "islandwar.smp"
+  local selectedmap = "islandwar.map"
+  local d
 
   menu = BosMenu(_("Start Game"))
 
@@ -340,14 +481,18 @@ function RunStartGameMenu(s)
   
   ResetMapOptions()
   menu:writeText(_("Difficulty:"), sx, sy*11)
-  menu:addDropDown({_("easy"), _("normal"), _("hard")}, sx + 90, sy*11,
+  d = menu:addDropDown({_("easy"), _("normal"), _("hard")}, sx + 150, sy*11,
     function(dd) GameSettings.Difficulty = (5 - dd:getSelected()*2) end)
-  menu:writeText(_("Map richness:"), sx, sy*11+25)
-  menu:addDropDown({_("high"), _("normal"), _("low")}, sx + 110, sy*11+25,
-    function(dd) GameSettings.MapRichness = (5 - dd:getSelected()*2) end)
-  menu:writeText(_("Starting resources:"), sx, sy*11+50)
-  menu:addDropDown({_("high"), _("normal"), _("low")}, sx + 150, sy*11+50,
+  d:setSelected(1)
+  menu:writeText(_("Starting resources:"), sx, sy*11+25)
+  d = menu:addDropDown({_("high"), _("normal"), _("low")}, sx + 150, sy*11+25,
     function(dd) GameSettings.Resources = (5 - dd:getSelected()*2) end)
+  d:setSelected(1)
+  menu:writeText(_("Game type:"), sx, sy*11+50)
+  d = menu:addDropDown({_("Map Default"), _("Melee"), _("Free For All"), _("Top vs Bottom"), _("Left vs Right"), _("Man vs Machine")}, sx + 150, sy*11+50,
+    function(dd) GameSettings.GameType = dd:getSelected() - 1 end)
+  d:setSelected(0)
+  d:setSize(140, d:getHeight())
 
   local OldPresentMap = PresentMap
   PresentMap = function(description, nplayers, w, h, id)
@@ -358,11 +503,12 @@ function RunStartGameMenu(s)
     OldPresentMap(description, nplayers, w, h, id)
   end
  
-  Load("maps/"..selectedmap)
-  local browser = menu:addBrowser("maps/", "^.*%.smp$",  sx*10, sy*2+20, sx*8, sy*11)
+  Load("maps/"..selectedmap..'/presentation.smp')
+  local browser = menu:addMapBrowser("maps/", sx*10, sy*2+20, sx*8, sy*11,
+                                     "maps/"..selectedmap)
   local function cb(s)
     maptext:setCaption(browser:getSelectedItem())
-    Load("maps/" .. browser:getSelectedItem())
+    Load("maps/" .. browser:getSelectedMap())
     selectedmap = browser:getSelectedItem()
   end
   browser:setActionCallback(cb)
@@ -370,13 +516,14 @@ function RunStartGameMenu(s)
   AllowAllUnits()
   local function startgamebutton(s)
     print("Starting map -------")
-    RunMap("maps/" .. selectedmap, nil, fow:isMarked(), revealmap:isMarked())
+    RunMap("maps/" .. browser:getSelectedMap(), nil, fow:isMarked(),
+           revealmap:isMarked())
     PresentMap = OldPresentMap
     menu:stop()
   end
   menu:addButton(_("~!Main Menu"), "m", Video.Width / 2 - 250, Video.Height - 100,
                  function() menu:stop() end)
-  menu:addButton(_("~!Start"), "s", Video.Width /2 + 50 ,  Video.Height - 100,
+  menu:addButton(_("~!Start"), "s", Video.Width / 2 + 50 ,  Video.Height - 100,
                  startgamebutton)
 
   menu:run()
@@ -393,7 +540,8 @@ function RunReplayMenu(s)
   -- Stratagus should store complete starting conditions in the log.
   AllowAllUnits()
 
-  local browser = menu:addBrowser("~logs/", ".log$", 300, 100, 300, 200)
+  local lister = CreateFilteringLister(".log$",  ListFilesInDirectory)
+  local browser = menu:addBrowser("~logs/", lister, 300, 100, 300, 200)
   local reveal = menu:addCheckBox(_("Reveal map"), 100, 250, function() end)
 
   function startreplaybutton(s)
@@ -407,7 +555,7 @@ function RunReplayMenu(s)
 
   menu:addButton(_("~!Main Menu"), "m", Video.Width / 2 - 250, Video.Height - 100,
                  function() menu:stop() end)
-  menu:addButton(_("~!Start"), "s", Video.Width /2 + 50 ,  Video.Height - 100,
+  menu:addButton(_("~!Start"), "s", Video.Width / 2 + 50 ,  Video.Height - 100,
                  startreplaybutton)
 
   menu:run()
@@ -419,15 +567,18 @@ function RunLoadGameMenu(s)
   local b
 
   menu = BosMenu(_("Load Game"))
-  local browser = menu:addBrowser("~save", ".sav.gz$", 
+  local lister = CreateFilteringLister(".sav.gz$",  ListFilesInDirectory)
+  local browser = menu:addBrowser("~save", lister, 
                                  Video.Width / 2 - 150, 100, 300, 200)
-    function startgamebutton(s)
+  function startgamebutton(s)
       print("Starting saved game")
       currentCampaign = nil
       loop = true
       while (loop) do
         InitGameVariables()
+        SavedGame = true
         StartSavedGame("~save/" .. browser:getSelectedItem())
+        SavedGame = false
         if (GameResult ~= GameRestart) then
           loop = false
         end
@@ -448,7 +599,7 @@ function RunLoadGameMenu(s)
   end
   menu:addButton(_("~!Main Menu"), "m", Video.Width / 2 - 250, Video.Height - 100,
                  function() menu:stop() end)
-  menu:addButton(_("~!Start"), "s", Video.Width /2 + 50 ,  Video.Height - 100,
+  menu:addButton(_("~!Start"), "s", Video.Width / 2 + 50 ,  Video.Height - 100,
                  startgamebutton)
 
   DisallowAllUnits()
@@ -456,10 +607,77 @@ function RunLoadGameMenu(s)
 end
 
 function RunEditorMenu(s)
+  local menu
+  local x = Video.Width / 2 - 100
+
+  menu = BosMenu(_("Editor"))
+
+  menu:addButton(_("Create ~!New Map"), "n", x, 260,
+    function() RunEditorNewMenu(); menu:stop() end)
+  menu:addButton(_("~!Load Map"), "l", x, 300,
+    function() RunEditorLoadMenu(); menu:stop() end)
+
+  menu:addButton(_("~!Cancel"), "c", x, Video.Height - 100,
+    function() menu:stop() end)
+
+  menu:run()
+end
+
+function RunEditorNewMenu()
+  local menu
+  local sy = Video.Height / 20
+  local xsize
+  local ysize
+  local defaultSize = {64, 64}
+
+  menu = BosMenu(_("Editor"))
+
+  function starteditorbutton(s)
+    -- FIXME: select tileset or image based map
+    LoadTileModels("scripts/tilesets/desert.lua")
+
+    local n = tonumber(xsize:getText())
+    if (n == nil) then n = 64 end
+    Map.Info.MapWidth = n
+    local n = tonumber(ysize:getText())
+    if (n == nil) then n = 64 end
+    Map.Info.MapHeight = n
+
+    StartEditor(nil)
+    Load("scripts/uilayout.lua")
+    HandleCommandKey = HandleIngameCommandKey
+    menu:stop()
+  end
+
+  local l = Label(_("Size:"))
+  l:setFont(Fonts["game"])
+  l:adjustSize()
+  menu:add(l, Video.Width / 2 - 50, 8 * sy)
+
+  xsize = menu:addTextInputField(tostring(defaultSize[1]),
+    Video.Width / 2 + 10, 8 * sy, 40)
+
+  local l = Label("x")
+  l:setFont(Fonts["game"])
+  l:adjustSize()
+  menu:add(l, Video.Width / 2 + 57, 8 * sy)
+
+  ysize = menu:addTextInputField(tostring(defaultSize[2]),
+    Video.Width / 2 + 10 + 60, 8 * sy, 40)
+
+  menu:addButton(_("~!Main Menu"), "m", Video.Width / 2 - 250, Video.Height - 100,
+    function() menu:stop() end)
+  menu:addButton(_("Start ~!Editor"), "e", Video.Width / 2 + 50, Video.Height - 100,
+    starteditorbutton)
+
+  menu:run()
+end
+
+function RunEditorLoadMenu()
   local menu
   local sx = Video.Width / 20
   local sy = Video.Height / 20
-  local selectedmap = "islandwar.smp"
+  local selectedmap = "islandwar.map"
   local numplayers = 2
 
   menu = BosMenu(_("Editor"))
@@ -483,20 +701,19 @@ function RunEditorMenu(s)
     OldPresentMap(description, nplayers, w, h, id)
   end
 
-  Load("maps/"..selectedmap)
-  local browser = menu:addBrowser("maps/", "^.*%.smp$", sx*10, sy*2+20, sx*8, sy*11)
+  Load("maps/"..selectedmap..'/presentation.smp')
+  local browser = menu:addMapBrowser("maps/", sx*10, sy*2+20, sx*8, sy*11, 
+                                     "maps/"..selectedmap)
   local function selectMap(s)
     maptext:setCaption(browser:getSelectedItem())
-    Load("maps/" .. browser:getSelectedItem())
+    Load("maps/" .. browser:getSelectedMap())
     selectedmap = browser:getSelectedItem()
   end
   browser:setActionCallback(selectMap)
 
   function starteditorbutton(s)
-    UI.MenuButton:SetCallback(function() RunEditorIngameMenu() end)
-    HandleCommandKey = HandleEditorIngameCommandKey
-    StartEditor(selectedmap)
-    UI.MenuButton:SetCallback(function() RunGameMenu() end)
+    StartEditor(browser:getSelectedMap())
+    Load("scripts/uilayout.lua")
     HandleCommandKey = HandleIngameCommandKey
     menu:stop()
   end
@@ -512,7 +729,6 @@ end
 Load("scripts/menus/network.lua")
 Load("scripts/menus/options.lua")
 Load("scripts/menus/credits.lua")
-Load("scripts/menus/widgetsdemo.lua")
 Load("scripts/menus/ingame/game.lua")
 Load("scripts/menus/ingame/editor.lua")
 Load("scripts/menus/campaigns.lua")
